#include "gpnd.h"



int appGetID(GPND_MAIN *m, const char *app) {
	int i, entry;
	i = 0;
	entry = -1;

	do {
		if (m->app[i].flags & GPND_MAIN_ENTRYISVALID && !(m->app[i].flags & GPND_MAIN_SKIPENTRY))
			if (strcmp(m->app[i].appname, app) == 0)
				entry = i;
		i++;
	} while (m->app[i - 1].flags & GPND_MAIN_THEREISNEXTENTRY);

	return entry;
}


void appFree(GPND_MAIN *m, int app) {
	int i;
	
	free(m->app[app].author);
	free(m->app[app].mail);
	free(m->app[app].http);
	free(m->app[app].icon);
	free(m->app[app].uid);
	free(m->app[app].appdata);
	free(m->app[app].appname);
	free(m->app[app].startdir);
	free(m->app[app].execfile);
	free(m->app[app].args);

	i = 0;

	do {
		if (m->app[app].ndesc[i].flags & GPND_MAIN_ENTRYISVALID) {
			free(m->app[app].ndesc[i].lang);
			free(m->app[app].ndesc[i].name);
			free(m->app[app].ndesc[i].desc);
		}
		i++;
	} while (m->app[app].ndesc[i - 1].flags & GPND_MAIN_THEREISNEXTENTRY);

	free(m->app[app].ndesc);
	i = 0;

	do {
		if (m->app[app].cat[i].flags & GPND_MAIN_ENTRYISVALID) {
			free(m->app[app].cat[i].subcat);
		}
		i++;
	} while (m->app[app].cat[i - 1].flags & GPND_MAIN_THEREISNEXTENTRY);

	free(m->app[app].cat);

	return;
}
			


int appAdd(GPND_MAIN *m, const char *app) {
	int i = -1;

	if (appGetID(m, (char *) app) != -1) {
		guiShowErrorMessage(m, "It does alredy exist a app with that name!");
		return i;
	}
	
	if (strlen(app) == 0) {
		guiShowErrorMessage(m, "You must enter a name for the app!");
		return i;
	}
	

	for (i = 0; (m->app[i].flags & GPND_MAIN_THEREISNEXTENTRY) && (m->app[i].flags & GPND_MAIN_ENTRYISVALID); i++);
	
	if (m->app[i].flags & GPND_MAIN_ENTRYISVALID) {
		m->app[i].flags |= GPND_MAIN_THEREISNEXTENTRY;
		i++;
		if ((m->app = realloc(m->app, sizeof(GPND_APP) * (i + 1))) == NULL) {
			guiShowErrorMessage(m, "Out of RAM!");
			exit(255);
		}

		m->app[i].flags = GPND_MAIN_ENTRYISVALID;
	} else
		m->app[i].flags |= GPND_MAIN_ENTRYISVALID;
	
	
	m->app[i].author = NULL;
	m->app[i].mail = NULL;
	m->app[i].http = NULL;
	m->app[i].icon = NULL;
	m->app[i].uid = NULL;
	m->app[i].appdata = NULL;
	m->app[i].appname = NULL;
	m->app[i].startdir = NULL;
	m->app[i].execfile = NULL;
	m->app[i].args = NULL;
	m->app[i].cat = 0;
	m->app[i].cpufreq = 0;
	m->app[i].allowMultitasking = 1;
	m->app[i].standalone = 0;
	m->app[i].x11 = 2;
	m->app[i].version[0] = 1;
	m->app[i].version[1] = 0;
	m->app[i].version[2] = 0;
	m->app[i].version[3] = 0;

	m->app[i].appname = realloc(m->app[i].appname, strlen(app) + 1);
	if (m->app[i].appname != NULL)
		sprintf(m->app[i].appname, "%s", app);
		
	m->app[i].ndesc = malloc(sizeof(GPND_NAMEDESC_STRUCT));
	m->app[i].ndesc->flags = 0;
	gpndInitCat(m, i);

	
	m->noofApps++;
	guiUpdateMainButtonActivity(m);
	
	return i;
}


int appMod(GPND_MAIN *m, const char *newname, int id) {
	
	if (id == -1) {
		guiShowErrorMessage(m, "No app is selected!");
		return -1;
	}
	
	m->app[id].flags |= GPND_MAIN_SKIPENTRY;
	
	if (appGetID(m, newname) != -1) {
		guiShowErrorMessage(m, "It does alredy exist a app with that name!");
		m->app[id].flags ^= GPND_MAIN_SKIPENTRY;
		return -1;
	}
	
	if (strlen(newname) > 0);
	else {
		guiShowErrorMessage(m, "You must enter a name for the app!");
		m->app[id].flags ^= GPND_MAIN_SKIPENTRY;
		return -1;
	}

	m->app[id].flags ^= GPND_MAIN_SKIPENTRY;

	if ((m->app[id].appname = realloc(m->app[id].appname, strlen(newname) + 1)) == NULL) {
		guiShowErrorMessage(m, "Out of RAM!");
		exit(255);
	}

	sprintf(m->app[id].appname, "%s", newname);

	return 0;
}


int appDel(GPND_MAIN *m) {
	if (m->activeApp == -1) 
		return -1;
	
	m->app[m->activeApp].flags ^= GPND_MAIN_ENTRYISVALID;
	appFree(m, m->activeApp);
	m->activeApp = -1;
	m->noofApps--;
	guiUpdateMainButtonActivity(m);

	return 0;
}
	
