#include "gpnd.h"


int generalSet(GPND_MAIN *m, GPND_GENERAL_VALIDATE_INPUT_STRUCT *val) {
	m->app[m->activeApp].author = realloc(m->app[m->activeApp].author, strlen(val->author) + 1);
	if (m->app[m->activeApp].author != NULL) {
		sprintf(m->app[m->activeApp].author, "%s", val->author);
		gpndEscapeXMLData(m->app[m->activeApp].author);
	}
	
	m->app[m->activeApp].http = realloc(m->app[m->activeApp].http, strlen(val->http) + 1);
	if (m->app[m->activeApp].http != NULL) {
		sprintf(m->app[m->activeApp].http, "%s", val->http);
		gpndEscapeXMLData(m->app[m->activeApp].http);
	}

	m->app[m->activeApp].mail = realloc(m->app[m->activeApp].mail, strlen(val->mail) + 1);
	if (m->app[m->activeApp].mail != NULL) {
		sprintf(m->app[m->activeApp].mail, "%s", val->mail);
		gpndEscapeXMLData(m->app[m->activeApp].mail);
	}

	m->app[m->activeApp].icon = realloc(m->app[m->activeApp].icon, strlen(val->icon) + 1);
	if (m->app[m->activeApp].icon != NULL) {
		sprintf(m->app[m->activeApp].icon, "%s", val->icon);
		gpndEscapeXMLData(m->app[m->activeApp].icon);
	}

	m->app[m->activeApp].version[0] = val->version[0];
	m->app[m->activeApp].version[1] = val->version[1];
	m->app[m->activeApp].version[2] = val->version[2];
	m->app[m->activeApp].version[3] = val->version[3];

	m->app[m->activeApp].flags |= GPND_GUI_GENERAL_INFO_IS_SET;

	return 0;
}


int generalValidate(GPND_MAIN *m, GPND_GENERAL_VALIDATE_INPUT_STRUCT *val) {
	char *temp;
	
	if (strlen(val->author) == 0) {
		guiShowErrorMessage(m, "You must specify the author!");
		return -1;
	}

	if (strlen(val->http) > 0) {
		if (strstr(val->http, "http://") != val->http) {
			guiShowErrorMessage(m, "If you specify a home page, it should begin with »http://«");
			return -1;
		} else if (strstr(val->http, ".") == NULL) {
			guiShowErrorMessage(m, "A home page must have a TLD...");
			return -1;
		}
	}

	if (strlen(val->http) > 0) {
		if (strstr(val->mail, "@") == NULL) {
			guiShowErrorMessage(m, "If you enter a mail address, it must be valid!");
			return -1;
		}

		else if (strstr(strstr(val->mail, "@"), ".") == NULL) {
			guiShowErrorMessage(m, "If you enter a mail address, it must be valid!");
			return -1;
		}
	}

	if (strlen(val->icon) > 0) {
		if (!gpndPathIsRelative(val->icon)) {
			if (!gpndFileExists(val->icon)) {
				guiShowErrorMessage(m, "Icon file does not exist!");
				return -1;
			}
		} else {
			temp = malloc(strlen(m->dirname) + strlen(val->icon) + 2);
			sprintf(temp, "%s/%s", m->dirname, val->icon);
			if (!gpndFileExists(temp)) {
				guiShowErrorMessage(m, "Icon file does not exist!");
				free(temp);
				return -1;
			}
			free(temp);
		}
	}

	if (val->version[0] > 255 || val->version[0] < 0 || val->version[1] > 255 || val->version[1] < 0 || val->version[2] > 255 || val->version[2] < 0 || val->version[3] > 255 || val->version[3] < 0) {
		guiShowErrorMessage(m, "Version numbers must be integers between 0 and 255!");
		return -1;
	}

	return 0;
}
