#include "gpnd.h"


int localeIsValid(GPND_MAIN *m, const char *loc_) {
	int i, loc;

	for (i = 0, loc = -1; loc == -1 && i < m->localelen; i++) {
		if (strcmp(m->locales[i], loc_) == 0)
			loc = i;
	}

	return loc;
}


int localeExists(GPND_MAIN *m, const char *locale) {
	int i, loc;

	i = 0;
	loc = -1;
	
	do {
		if (m->app[m->activeApp].ndesc[i].flags & GPND_MAIN_ENTRYISVALID) {
			if (strcmp(m->app[m->activeApp].ndesc[i].lang, locale) == 0 && !(m->app[m->activeApp].ndesc[i].flags & GPND_MAIN_SKIPENTRY)) {
				loc = i;
				return i;
			}
		}
		
		i++;
	} while (m->app[m->activeApp].ndesc[i-1].flags & GPND_MAIN_THEREISNEXTENTRY);

	return loc;
}


int localeValidate(GPND_MAIN *m, GPND_LOCALE_VALIDATE_INPUT_STRUCT *val) {

	if (strlen(val->locale) == 0) {
		guiShowErrorMessage(m, "You must enter a locale!");
		return -1;
	}
	
	if (localeIsValid(m, val->locale) == -1) {
		guiShowErrorMessage(m, "Entered locale does not exist!");
		return -1;
	}

	if (localeExists(m, val->locale) != -1) {
		guiShowErrorMessage(m, "It does alredy exist a entry for that locale!");
		return -1;
	}

	if (strlen(val->name) == 0) {
		guiShowErrorMessage(m, "You must enter a name for the application!");
		return -1;
	}

	if (strlen(val->desc) == 0) {
		guiShowErrorMessage(m, "You must enter a description for the application!");
		return -1;
	}

	return 0;
}


int localeAdd(GPND_MAIN *m, GPND_LOCALE_VALIDATE_INPUT_STRUCT *val) {
	int i;

	for (i = 0; (m->app[m->activeApp].ndesc[i].flags & GPND_MAIN_ENTRYISVALID) && (m->app[m->activeApp].ndesc[i].flags & GPND_MAIN_THEREISNEXTENTRY); i++);

	
	if (!(m->app[m->activeApp].ndesc[i].flags & GPND_MAIN_THEREISNEXTENTRY)) {
		if ((m->app[m->activeApp].ndesc = realloc(m->app[m->activeApp].ndesc, sizeof(GPND_NAMEDESC_STRUCT) * (i + 2))) == NULL) {
			guiShowErrorMessage(m, "Out of RAM");
			exit (255);
		}
		
		m->app[m->activeApp].ndesc[i+1].flags = 0;
		m->app[m->activeApp].ndesc[i].flags = GPND_MAIN_THEREISNEXTENTRY;
	}

	m->app[m->activeApp].ndesc[i].flags |= GPND_MAIN_ENTRYISVALID;

	if ((m->app[m->activeApp].ndesc[i].lang = malloc(sizeof(strlen(val->locale) + 1))) == NULL) {
		m->app[m->activeApp].ndesc[i].flags ^= GPND_MAIN_ENTRYISVALID;
		guiShowErrorMessage(m, "Out of RAM!");
		return -1;
	}
	sprintf(m->app[m->activeApp].ndesc[i].lang, "%s", val->locale);

	if ((m->app[m->activeApp].ndesc[i].name = malloc(sizeof(strlen(val->name) + 1))) == NULL) {
		free(m->app[m->activeApp].ndesc[i].lang);
		m->app[m->activeApp].ndesc[i].flags ^= GPND_MAIN_ENTRYISVALID;
		guiShowErrorMessage(m, "Out of RAM!");
		return -1;
	}
	sprintf(m->app[m->activeApp].ndesc[i].name, "%s", val->name);

	if ((m->app[m->activeApp].ndesc[i].desc = malloc(sizeof(strlen(val->desc) + 1))) == NULL) {
		free(m->app[m->activeApp].ndesc[i].lang);
		free(m->app[m->activeApp].ndesc[i].name);
		m->app[m->activeApp].ndesc[i].flags ^= GPND_MAIN_ENTRYISVALID;
		guiShowErrorMessage(m, "Out of RAM!");
		return -1;
	}
	sprintf(m->app[m->activeApp].ndesc[i].desc, "%s", val->desc);

	
	return 0;
}


int localeMod(GPND_MAIN *m, const char *curloc, GPND_LOCALE_VALIDATE_INPUT_STRUCT *val) {
	int i;
	
	if ((i = localeExists(m, curloc)) == -1) {
		return -1;
	}

	m->app[m->activeApp].ndesc[i].flags |= GPND_MAIN_SKIPENTRY;
	if (localeValidate(m, val) == -1) {
		return -1;
	}
	
	m->app[m->activeApp].ndesc[i].flags ^= GPND_MAIN_SKIPENTRY;
	
	free(m->app[m->activeApp].ndesc[i].lang);
	free(m->app[m->activeApp].ndesc[i].name);
	free(m->app[m->activeApp].ndesc[i].desc);

	
	if ((m->app[m->activeApp].ndesc[i].lang = malloc(sizeof(strlen(val->locale) + 1))) == NULL) {
		m->app[m->activeApp].ndesc[i].flags ^= GPND_MAIN_ENTRYISVALID;
		guiShowErrorMessage(m, "Out of RAM!");
		return -1;
	}
	sprintf(m->app[m->activeApp].ndesc[i].lang, "%s", val->locale);

	if ((m->app[m->activeApp].ndesc[i].name = malloc(sizeof(strlen(val->name) + 1))) == NULL) {
		free(m->app[m->activeApp].ndesc[i].lang);
		m->app[m->activeApp].ndesc[i].flags ^= GPND_MAIN_ENTRYISVALID;
		guiShowErrorMessage(m, "Out of RAM!");
		return -1;
	}
	sprintf(m->app[m->activeApp].ndesc[i].name, "%s", val->name);

	if ((m->app[m->activeApp].ndesc[i].desc = malloc(sizeof(strlen(val->desc) + 1))) == NULL) {
		free(m->app[m->activeApp].ndesc[i].lang);
		free(m->app[m->activeApp].ndesc[i].name);
		m->app[m->activeApp].ndesc[i].flags ^= GPND_MAIN_ENTRYISVALID;
		guiShowErrorMessage(m, "Out of RAM!");
		return -1;
	}
	sprintf(m->app[m->activeApp].ndesc[i].desc, "%s", val->desc);

	
	return 0;
}


int localeDel(GPND_MAIN *m, const char *curloc) {
	int i;

	if ((i = localeExists(m, curloc)) == -1) {
		return -1;
	}

	
	free(m->app[m->activeApp].ndesc[i].lang);
	free(m->app[m->activeApp].ndesc[i].name);
	free(m->app[m->activeApp].ndesc[i].desc);
	
	m->app[m->activeApp].ndesc[i].flags ^= GPND_MAIN_ENTRYISVALID;
	printf("Free() did okay...\n");
	
	return 0;
}
