#include "gpnd.h"


int runtimeSet(GPND_MAIN *m, GPND_RUNTIME_VALIDATE_INPUT_STRUCT *val) {
	
	m->app[m->activeApp].uid = realloc(m->app[m->activeApp].uid, strlen(val->uid) + 1);
	if (m->app[m->activeApp].uid != NULL) {
		sprintf(m->app[m->activeApp].uid, "%s", val->uid);
		gpndEscapeXMLData(m->app[m->activeApp].uid);
	}
	
	m->app[m->activeApp].appdata = realloc(m->app[m->activeApp].appdata, strlen(val->appdata) + 1);
	if (m->app[m->activeApp].appdata != NULL) {
		sprintf(m->app[m->activeApp].appdata, "%s", val->appdata);
		gpndEscapeXMLData(m->app[m->activeApp].appdata);
	}

	m->app[m->activeApp].startdir = realloc(m->app[m->activeApp].startdir, strlen(val->startdir) + 1);
	if (m->app[m->activeApp].startdir != NULL) {
		sprintf(m->app[m->activeApp].startdir, "%s", val->startdir);
		gpndEscapeXMLData(m->app[m->activeApp].startdir);
	}

	m->app[m->activeApp].execfile = realloc(m->app[m->activeApp].execfile, strlen(val->execfile) + 1);
	if (m->app[m->activeApp].execfile != NULL) {
		sprintf(m->app[m->activeApp].execfile, "%s", val->execfile);
		gpndEscapeXMLData(m->app[m->activeApp].execfile);
	}

	m->app[m->activeApp].args = realloc(m->app[m->activeApp].args, strlen(val->args) + 1);
	if (m->app[m->activeApp].args != NULL) {
		sprintf(m->app[m->activeApp].args, "%s", val->args);
		gpndEscapeXMLData(m->app[m->activeApp].args);
	}

	m->app[m->activeApp].cpufreq = val->cpufreq;
	m->app[m->activeApp].x11 = val->x11;
	m->app[m->activeApp].allowMultitasking = val->allowMultitasking;
	m->app[m->activeApp].standalone = val->standalone;

	m->app[m->activeApp].flags |= GPND_GUI_RUNTIME_INFO_IS_SET;

	return 0;
}


int runtimeValidate(GPND_MAIN *m, GPND_RUNTIME_VALIDATE_INPUT_STRUCT *val) {
	char *temp;

	if (strlen(val->uid) == 0) {
		guiShowErrorMessage(m, "You must specify a unique ID!");
		return -1;
	}

	if (strlen(val->execfile) == 0) {
		guiShowErrorMessage(m, "You must specify the application binary!");
		return -1;
	}

	if (strlen(val->execfile) > 0) {
		if (!gpndPathIsRelative(val->execfile)) {
			if (!gpndFileExists(val->execfile)) {
				guiShowErrorMessage(m, "Application binary does not exist!");
				return -1;
			}
		} else {
			temp = malloc(strlen(m->dirname) + strlen(val->execfile) + 2);
			sprintf(temp, "%s/%s", m->dirname, val->execfile);
			if (!gpndFileExists(temp)) {
				guiShowErrorMessage(m, "Application binary does not exist!");
				free(temp);
				return -1;
			}
			free(temp);
		}
	}

	if (strlen(val->appdata) > 0) {
		if (gpndArgIsPath(val->appdata)) {
			guiShowErrorMessage(m, "Appdata directory must be a name, not a path!");
			return -1;
		}
	}

	return 0;
}
